/*

Copyright (c) 2005-2014, University of Oxford.
All rights reserved.

University of Oxford means the Chancellor, Masters and Scholars of the
University of Oxford, having an administrative office at Wellington
Square, Oxford OX1 2JD, UK.

This file is part of Chaste.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of the University of Oxford nor the names of its
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef RANGEHELPERFUNCTIONS_HPP_
#define RANGEHELPERFUNCTIONS_HPP_

#include <string>
#include <map>
#include <vector>

#include "AbstractValue.hpp"
#include "NdArray.hpp"

// These utility functions are used by ArrayCreate and View.

/**
 * Check the start, end, or dimension of a range specification.
 *
 * @param pValue  the value specifying the start/end/dimension
 * @param nullOk  whether the null value is allowed here
 * @param specNum  which range specification this is
 * @param rLocationInfo  source program location to use in error messages
 * @param shouldBeIndex  whether the return value should fit within the NdArray<double>::Index type too
 */
NdArray<double>::RangeIndex CheckRangeElement(const AbstractValuePtr pValue,
                                              bool nullOk,
                                              NdArray<double>::Index specNum,
                                              const std::string& rLocationInfo,
                                              bool shouldBeIndex=false);

/**
 * Extract range specifications from a vector of tuple values.
 *
 * The tuple values should contain (dimension, start, stride, end, name) if pRangeNames
 * is supplied.  If not, they may contain either the first 3 or 4 items, but must all have
 * the same length.  If only 3 items are used, the dimension is taken to be the index into
 * the vector.
 *
 * @param rRangeSpecs  to be filled in with the range specifications
 * @param rTuples  the tuple values
 * @param rLocationInfo  source program location to use in error messages
 * @param pRangeNames  to be filled in with the names to use for index variables, if given
 */
void ExtractRangeSpecs(std::map<NdArray<double>::Index, NdArray<double>::Range>& rRangeSpecs,
                       const std::vector<AbstractValuePtr>& rTuples,
                       const std::string& rLocationInfo,
                       std::map<NdArray<double>::Index, std::string>* pRangeNames=NULL);

#endif // RANGEHELPERFUNCTIONS_HPP_
